From bf4b332d7b481549c6d9cfa70db51e39a305b9b2 Mon Sep 17 00:00:00 2001
From: Anthonin Bonnefoy <anthonin.bonnefoy@datadoghq.com>
Date: Wed, 17 Jul 2024 10:27:35 +0200
Subject: [PATCH] Report query_id in ExecutorRun for extended protocol queries

When using extended protocol, exec_execute_message will reset
MyBEEntry's query_id when calling pgstat_report_activity. The query_id
will never be set after that since it is only set in parse analysis and
ExecutorStart. Because of that, all queries executed through extended
protocol will report an empty query_id in pg_stat_activity and the
query_id won't be propagated to parallel workers.

The patch fixes the issue by setting query_id in ExecutorRun, similarly
to what is already done in ExecutorStart.
---
 src/backend/executor/execMain.c | 8 ++++++++
 src/backend/tcop/utility.c      | 7 +++++++
 2 files changed, 15 insertions(+)

diff --git a/src/backend/executor/execMain.c b/src/backend/executor/execMain.c
index 4d7c92d63c198..69cca12723923 100644
--- a/src/backend/executor/execMain.c
+++ b/src/backend/executor/execMain.c
@@ -298,6 +298,14 @@ ExecutorRun(QueryDesc *queryDesc,
 			ScanDirection direction, uint64 count,
 			bool execute_once)
 {
+	/*
+	 * When using extended protocol, query_id will be reset during
+	 * pgstat_report_activity and never set since we don't go through parse
+	 * analysis or ExecutorStart. Similar to what's done in ExecutorStart,
+	 * report the queryId we know from the queryDesc.
+	 */
+	pgstat_report_query_id(queryDesc->plannedstmt->queryId, false);
+
 	if (ExecutorRun_hook)
 		(*ExecutorRun_hook) (queryDesc, direction, count, execute_once);
 	else
diff --git a/src/backend/tcop/utility.c b/src/backend/tcop/utility.c
index fa66b8017edea..8cbd95701d93f 100644
--- a/src/backend/tcop/utility.c
+++ b/src/backend/tcop/utility.c
@@ -63,6 +63,7 @@
 #include "storage/fd.h"
 #include "tcop/utility.h"
 #include "utils/acl.h"
+#include "utils/backend_status.h"
 #include "utils/guc.h"
 #include "utils/lsyscache.h"
 
@@ -510,6 +511,12 @@ ProcessUtility(PlannedStmt *pstmt,
 	Assert(queryString != NULL);	/* required as of 8.4 */
 	Assert(qc == NULL || qc->commandTag == CMDTAG_UNKNOWN);
 
+	/*
+	 * Utility statement executed through extended protocol won't have
+	 * query_id set, report it here
+	 */
+	pgstat_report_query_id(pstmt->queryId, false);
+
 	/*
 	 * We provide a function hook variable that lets loadable plugins get
 	 * control when ProcessUtility is called.  Such a plugin would normally
